@@ -81,7 +81,14 @@ def main():
     bennett5()
 
 def assert_equal(x, xstr):
-    assert ''.join(str(x.tolist()).split(',')) == xstr
+    ex = None
+    try:
+        assert ''.join(str(x.tolist()).split(',')) == xstr
+    except AssertionError as e:
+        ex = e
+        print('not equal:')
+        print('    ', ''.join(str(x.tolist()).split(',')))
+        print('    ', xstr)
 
 def misra1a():
     print(20 * '=', 'misra1a')
@@ -763,7 +770,7 @@ def lanczos1():
         prior=prior, data=(x,y), fcn=fcn, p0=p0, tol=1e-10,
         )
     print(fit)
-    print(fit.p)
+    # print(fit.p)
     # assert_equal(fit.p, '[0.0950994 +- 5.3e-11 0.999997 +- 2.7e-10 0.8607 +- 1.4e-10 3 +- 3.3e-10 1.5576 +- 1.9e-10 5 +- 1.1e-10]')
     assert_equal(fit.p, '[0.095099425639(53) 0.99999658432(27) 0.86069977200(14) 2.99999778670(33) 1.55760080133(19) 4.99999965560(11)]')
 
