@@ -100,6 +100,8 @@ def empbayes_fit(z0, fitargs, **minargs):
     fit curve to estimate that the uncertainty in each data point is
     of order 1.6%.
 
+    See also :meth:`MultiFitter.empbayes_fit`.
+
     Args:
         z0 (number, array or dict): Starting point for search.
         fitargs (callable): Function of ``z`` that returns a
@@ -123,23 +125,25 @@ def empbayes_fit(z0, fitargs, **minargs):
         :class:`lsqfit.nonlinear_fit`) and the
         optimal value for parameter ``z``.
     """
+    return _empbayes_fit(z0, fitargs, nonlinear_fit=lsqfit.nonlinear_fit, **minargs)
+
+def _empbayes_fit(z0, fitargs, nonlinear_fit, **minargs):
+    " For use by lsqfit.empbayes_fit and lsqfit.MultiFitter.empbayes_fit "
     save = dict(lastz=None, lastp0=None)
     if hasattr(z0, 'keys'):
         # z is a dictionary
-        if not isinstance(z0, gvar.BufferDict):
-            z0 = gvar.BufferDict(z0)
+        z0 = gvar.BufferDict(z0, dtype=float)
         z0buf = z0.buf
         def convert(zbuf):
             return gvar.BufferDict(z0, buf=zbuf)
     elif numpy.shape(z0) == ():
         # z is a number
-        z0buf = numpy.array([z0])
+        z0buf = numpy.array([z0], dtype=float)
         def convert(zbuf):
             return zbuf[0]
     else:
         # z is an array
-        z0 = numpy.asarray(z0)
-        z0buf = z0
+        z0buf = numpy.asarray(z0, dtype=float)
         def convert(zbuf):
             return zbuf
     def minfcn(zbuf, save=save, convert=convert):
@@ -151,7 +155,7 @@ def empbayes_fit(z0, fitargs, **minargs):
             plausibility = 0.0
         if save['lastp0'] is not None:
             args['p0'] = save['lastp0']
-        fit = lsqfit.nonlinear_fit(**args)
+        fit = nonlinear_fit(**args)
         if numpy.isnan(fit.logGBF):
             raise ValueError
         else:
@@ -168,7 +172,7 @@ def empbayes_fit(z0, fitargs, **minargs):
         args, plausibility = args
     if save['lastp0'] is not None:
         args['p0'] = save['lastp0']
-    return lsqfit.nonlinear_fit(**args), z
+    return nonlinear_fit(**args), z
 
 
 class GVarWAvg(gvar.GVar):
@@ -199,7 +203,7 @@ class GVarWAvg(gvar.GVar):
 
         Time required to do average.
 
-    .. attribute:: svdcorrection
+    .. attribute:: correction
 
         The *svd* corrections added to the data in the average.
 
@@ -213,7 +217,7 @@ class GVarWAvg(gvar.GVar):
             self.chi2 = 0
             self.dof = 1
             self.Q = 1
-            self.svdcorrection = gvar.gvar(0,0)
+            self.correction = gvar.gvar(0,0)
             self.time = 0
             self.fit = None
         else:
@@ -221,8 +225,9 @@ class GVarWAvg(gvar.GVar):
             self.dof = fit.dof
             self.Q = fit.Q
             self.time = fit.time
-            self.svdcorrection = fit.svdcorrection
+            self.correction = fit.correction
             self.fit = fit
+        self.svdcorrection = self.correction # legacy name
 
 class ArrayWAvg(numpy.ndarray):
     """ Result from weighted average :func:`lsqfit.wavg`.
@@ -252,7 +257,7 @@ class ArrayWAvg(numpy.ndarray):
 
         Time required to do average.
 
-    .. attribute:: svdcorrection
+    .. attribute:: correction
 
         The *svd* corrections added to the data in the average.
 
@@ -267,13 +272,13 @@ class ArrayWAvg(numpy.ndarray):
             obj.dof = 1
             obj.Q = 1
             obj.time = 0
-            obj.svdcorrection = gvar.gvar(0, 0)
+            obj.correction = gvar.gvar(0, 0)
         else:
             obj.chi2 = fit.chi2
             obj.dof = fit.dof
             obj.Q = fit.Q
             obj.time = fit.time
-            obj.svdcorrection = fit.svdcorrection
+            obj.correction = fit.correction
             obj.fit = fit
         return obj
 
@@ -314,7 +319,7 @@ class BufferDictWAvg(gvar.BufferDict):
 
         Time required to do average.
 
-    .. attribute:: svdcorrection
+    .. attribute:: correction
 
         The *svd* corrections added to the data in the average.
 
@@ -328,7 +333,7 @@ class BufferDictWAvg(gvar.BufferDict):
             self.chi2 = 0
             self.dof = 1
             self.Q = 1
-            self.svdcorrection = gvar.gvar(0,0)
+            self.correction = gvar.gvar(0,0)
             self.time = 0
             self.fit = None
         else:
@@ -336,8 +341,9 @@ class BufferDictWAvg(gvar.BufferDict):
             self.dof = fit.dof
             self.Q = fit.Q
             self.time = fit.time
-            self.svdcorrection = fit.svdcorrection
+            self.correction = fit.correction
             self.fit = fit
+        self.svdcorrection = self.correction  # legacy name
 
 def wavg(datalist, fast=False, prior=None, **fitterargs):
     """ Weighted average of |GVar|\s or arrays/dicts of |GVar|\s.
@@ -389,8 +395,9 @@ def wavg(datalist, fast=False, prior=None, **fitterargs):
             sequentially. This can be much faster when averaging a large
             number of sampes but is only approximate if the different
             elements of ``datalist`` are correlated. Default is ``False``.
-        fitterargs (dict): Additional arguments (e.g., ``svdcut``) for the
-            :class:`lsqfit.nonlinear_fit` fitter used to do the averaging.
+        fitterargs (dict): Additional arguments (e.g., ``svdcut`` or ``eps``) 
+            for the :class:`lsqfit.nonlinear_fit` fitter used to do the 
+            averaging.
 
     Returns:
         The weighted average is returned as a |GVar| or an array of
@@ -410,8 +417,8 @@ def wavg(datalist, fast=False, prior=None, **fitterargs):
 
         **time** - Time required to do average.
 
-        **svdcorrection** - The *svd* corrections made to the data
-            when ``svdcut`` is not ``None``.
+        **correction** - The corrections made to the data
+            when ``svdcut>0`` or ``eps>0``.
 
         **fit** - Fit returned by :class:`lsqfit.nonlinear_fit`.
     """
@@ -431,19 +438,19 @@ def wavg(datalist, fast=False, prior=None, **fitterargs):
         else:
             return ArrayWAvg(numpy.asarray(datalist[0]), None)
     if fast:
-        chi2 = dof = time = svdcorrection = 0
+        chi2 = dof = time = correction = 0
         ans = datalist[0]
         for i, di in enumerate(datalist[1:]):
             ans = wavg([ans, di], fast=False, **fitterargs)
             chi2 += ans.chi2
             dof += ans.dof
             time += ans.time
-            svdcorrection += ans.svdcorrection
+            correction += ans.correction
         ans.fit.dof = dof
         ans.fit.Q = _gammaQ(dof / 2., chi2 / 2.)
         ans.fit.chi2 = chi2
         ans.fit.time = time
-        ans.fit.svdcorrection = svdcorrection
+        ans.fit.correction = correction
         return ans
     if hasattr(datalist[0], 'keys'):
         datashape = None
@@ -725,24 +732,25 @@ class chained_nonlinear_fit(lsqfit.nonlinear_fit):
         self.y = self.data
 
         self.linear = []
-        self.svdcorrection = 0
+        self.correction = 0
         self.svdn = 0
         self.dof = 0
         self.chi2 = 0
         self.nit = 0
         self.time = 0
-        self.svdcut = 0
+        self.svdcut = None
+        self.eps = None
         self.tol = [0., 0., 0.]
         self.error = []
         self.logGBF = 0.
-        self.add_svdnoise = False
+        self.noise = (False, False)
         self.fitter = 'chained fit'
         self.description = ''
         self.stopping_criterion = None
         self.residuals = []
         self.p0 = []
         for k in self.chained_fits:
-            self.svdcorrection += self.chained_fits[k].svdcorrection
+            self.correction += self.chained_fits[k].correction
             if k[:5] == 'wavg(' and k[-1] == ')':
                 continue
             self.residuals.extend(self.chained_fits[k].residuals)
@@ -753,10 +761,21 @@ class chained_nonlinear_fit(lsqfit.nonlinear_fit):
             self.time += self.chained_fits[k].time
             self.p0.append(self.chained_fits[k].p0)
             svdcut = self.chained_fits[k].svdcut
-            if self.chained_fits[k].add_svdnoise:
-                self.add_svdnoise = True
-            if svdcut is not None and  abs(svdcut) > abs(self.svdcut):
-                self.svdcut = svdcut
+            eps = self.chained_fits[k].eps
+            if self.chained_fits[k].noise[0]:
+                self.noise = (True, self.noise[1])
+            if self.chained_fits[k].noise[1]:
+                self.noise = (self.noise[0], True)
+            if svdcut is not None:
+                if self.svdcut is None:
+                    self.svdcut = svdcut
+                elif abs(svdcut) > abs(self.svdcut):
+                    self.svdcut = svdcut
+            if eps is not None:
+                if self.eps is None:
+                    self.eps = eps 
+                elif eps > self.eps:
+                    self.eps = eps
             tol = self.chained_fits[k].tol
             for i in range(3):
                 self.tol[i] = max(self.tol[i], tol[i])
@@ -780,6 +799,7 @@ class chained_nonlinear_fit(lsqfit.nonlinear_fit):
         self.cov = None
         self.fitter_results = None
         self.nblocks = None
+        self.svdcorrection = self.correction # legacy name
 
     def _remove_gvars(self, gvlist):
         self.p  # need to fill _p
@@ -921,7 +941,7 @@ class MultiFitter(object):
 
     def __init__(
         self, models, mopt=None, ratio=False, fast=True, wavg_all=False,
-        wavg_kargs=dict(svdcut=1e-12), fitname=None, fitterargs={},
+        wavg_kargs=dict(eps=1e-12), fitname=None, fitterargs={},
         **more_fitterargs
         ):
         super(MultiFitter, self).__init__()
@@ -1070,7 +1090,7 @@ class MultiFitter(object):
             ans = MultiFitter._flatten_models(tasklist)
         return ans
 
-    def lsqfit(self, data=None, pdata=None, prior=None, p0=None, **kargs):
+    def lsqfit(self, data=None, pdata=None, prior=None, p0=None, chained=False, **kargs):
         """ Compute least-squares fit of models to data.
 
         :meth:`MultiFitter.lsqfit` fits all of the models together, in
@@ -1107,11 +1127,17 @@ class MultiFitter(object):
                 the file with name ``"filename"`` for initial values and to
                 write out best-fit parameter values after the fit (for the
                 next call to ``self.lsqfit()``).
+            chained (bool): Use :meth:`MultiFitter.chained_lsqfit`
+                instead of :meth:`MultiFitter.lsqfit` if ``chained=True``. 
+                Ignored otherwise. Default is ``chained=False``.
             kargs: Arguments that (temporarily) override parameters specified
                 when the :class:`MultiFitter` was created. Can also include
                 additional arguments to be passed through to the :mod:`lsqfit`
                 fitter.
         """
+        # chained?
+        if chained:
+            return self.chained_lsqfit(data=data, pdata=pdata, prior=prior, p0=p0, **kargs)
         # gather parameters
         if prior is None:
             raise ValueError('no prior')
@@ -1186,7 +1212,7 @@ class MultiFitter(object):
         return self.fit
 
     def chained_lsqfit(
-        self, data=None, pdata=None, prior=None, p0=None, add_priornoise=None,
+        self, data=None, pdata=None, prior=None, p0=None,
         **kargs
         ):
         """ Compute chained least-squares fit of models to data.
@@ -1282,9 +1308,16 @@ class MultiFitter(object):
         """
         if prior is None:
             raise ValueError('no prior')
-        if add_priornoise:
-            # intercept at top level so same treatment for all fits
+        # prior is shared by many subfits so need to handle prior noise here
+        if 'noise' in kargs:
+            if isinstance(kargs['noise'], bool):
+                kargs['noise'] = (kargs['noise'], kargs['noise'])
+            if kargs['noise'][1]:
+                prior = prior + (gvar.sample(prior) - gvar.mean(prior))
+                kargs['noise'] = (kargs['noise'][0], False)
+        elif kargs.get('add_priornoise', False):
             prior = prior + (gvar.sample(prior) - gvar.mean(prior))
+            kargs['add_priornoise'] = False
         kargs, oldargs = self.set(**kargs)
 
         # parameters for bootstrap (see below)
@@ -1386,6 +1419,50 @@ class MultiFitter(object):
         self.set(**oldargs)
         return self.fit
 
+    def empbayes_fit(self, z0, fitargs, **minargs):
+        """ Return fit and ``z`` corresponding to the fit
+        ``self.lsqfit(**fitargs(z))`` that maximizes ``logGBF``.
+
+        This function maximizes the logarithm of the Bayes Factor from
+        fit  ``self.lsqfit(**fitargs(z))`` by varying ``z``,
+        starting at ``z0``. The fit is redone for each value of ``z``
+        that is tried, in order to determine ``logGBF``.
+
+        The Bayes Factor is proportional to the probability that the data
+        came from the model (fit function and priors) used in the fit.
+        :meth:`MultiFitter.empbayes_fit` finds the model or data that maximizes this
+        probability. See :func:`lsqfit.empbayes_fit` for more information.
+
+        Include ``chained=True`` in the dictionary returned by ``fitargs(z)``
+        if chained fits are desired. See documentation 
+        for :meth:`MultiFitter.lsqfit`.
+
+        Args:
+            z0 (number, array or dict): Starting point for search.
+            fitargs (callable): Function of ``z`` that returns a
+                dictionary ``args`` containing the :meth:`MultiFitter.lsqfit`
+                arguments corresponding to ``z``. ``z`` should have
+                the same layout (number, array or dictionary) as ``z0``.
+                ``fitargs(z)`` can instead return a tuple ``(args, plausibility)``,
+                where ``args`` is again the dictionary for
+                :meth:`MultiFitter.lsqfit`. ``plausibility`` is the logarithm
+                of the *a priori* probabilitiy that ``z`` is sensible. When
+                ``plausibility`` is provided, :func:`MultiFitter.empbayes_fit`
+                maximizes the sum ``logGBF + plausibility``. Specifying
+                ``plausibility`` is a way of steering selections away from
+                completely implausible values for ``z``.
+            minargs (dict): Optional argument dictionary, passed on to
+                :class:`lsqfit.gsl_multiminex` (or
+                :class:`lsqfit.scipy_multiminex`), which finds the minimum.
+
+        Returns:
+            A tuple containing the best fit (a fit object) and the
+            optimal value for parameter ``z``.
+        """
+        def nlinear_fit(**args):
+            return self.lsqfit(**args)
+        return _empbayes_fit(z0, fitargs, nonlinear_fit=nlinear_fit, **minargs)
+
     @staticmethod
     def _compile_models(models):
         """ Convert ``models`` into a list of tasks.
